# coding=utf-8
#
# __init__.py
# Fira
#
# Created by Marquis Kurt on 03/31/20.
# Copyright © 2020 Marquis Kurt. All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
#

"""The `uvn_fira` package contains all of the source code for the Unscripted minigame."""
from uvn_fira.api import *
from uvn_fira.core import *

__version__ = "1.2.0"
