# coding=utf-8
#
# __init__.py
# Fira Core
#
# Created by Marquis Kurt on 04/02/20.
# Copyright © 2020 Marquis Kurt. All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
#
"""The `core` module contains the utilities, functions, and classes that control the primary logic
    for the Unscripted minigame.
"""

from .config import *
from .data import *
from .grid import *
from .template import *
from .vm import *

__version__ = "1.2.0"
